import numpy as np
import PIL
from PIL import ImageFont
from PIL import Image
from PIL import ImageDraw
import matplotlib.pylab as plt
from io import BytesIO
import os


def crop_out_identical_margins(img, x_margin=0, y_margin=0):
    """
    Returns a cropped image of the input, where the "uninformative" margins of the images are cropped out.

    :param img: A PIL.Image.Image
    :return: A PIL.Image.Image where the margins that are identical (i.e. same exact "color+alpha") are cropped out
    """
    ima = np.asarray(img)
    if len(ima.shape) > 2:
        ima = ima.sum(axis=2)

    x_size = ima.shape[1]
    y_size = ima.shape[0]

    min_x = np.where(np.diff(ima.sum(axis=0)) != 0)[0][0]
    min_x -= min(min_x, x_margin)
    min_y = np.where(np.diff(ima.sum(axis=1)) != 0)[0][0]
    min_y -= min(min_y, y_margin)

    max_x = np.where(np.diff(ima.sum(axis=0)) != 0)[0][-1] + 1
    max_x += min(x_size - max_x, x_margin)
    max_y = np.where(np.diff(ima.sum(axis=1)) != 0)[0][-1] + 1
    max_y += min(y_size - max_y, y_margin)

    print((x_size, y_size))
    print((min_x, min_y, max_x, max_y))
    return img.crop((min_x, min_y, max_x, max_y))


def func_to_get_pil_images_like_this_one(x, **kwargs):
    """Get a function that will convert images, specified as the example input x, into PIL.Image types"""
    if isinstance(x, Image.Image):
        def to_pil_image(x):
            return x
    elif isinstance(x, plt.Figure):
        def to_pil_image(x):
            fp = BytesIO()
            kwargs['format'] = kwargs.get('format', 'jpg')
            x.savefig(fp, **kwargs)
            return Image.open(fp)
    else:
        to_pil_image = Image.open

    return to_pil_image


def x_to_pil_image(x, **kwargs):
    to_pil_image = func_to_get_pil_images_like_this_one(x, **kwargs)
    return to_pil_image(x)


def write_images(images, fp='test.pdf', pil_write_format=None, to_pil_image_kwargs=None, **pil_save_kwargs):
    """Write images or figs into pdf pages (one image per page)

    Args:
        images: iterable of "images" (should be a matplotlib figure, a PIL.Image,
            or a type that PIL.Image.open can open")
        fp: The filepath, path.Path, or file object to dump the images into
        **params: # extra parameters that are passed on to the save function

    Returns:

    """
    to_pil_image_kwargs = to_pil_image_kwargs or {}
    _, ext_format = os.path.splitext(fp)
    if ext_format.startswith('.'):
        ext_format = ext_format[1:]
    pil_write_format = pil_write_format or ext_format or 'PDF'
    im_iter = iter(images)
    first_im = next(im_iter)
    to_pil_image = func_to_get_pil_images_like_this_one(first_im, **to_pil_image_kwargs)
    return to_pil_image(first_im).save(fp, format=pil_write_format, save_all=True,
                                       append_images=map(to_pil_image, im_iter), **pil_save_kwargs)



__author__ = 'thor'


import ImageChops
import math


# The 2011 version using more recent Python idioms

def rmsdiff(im1, im2):
    "Calculate the root-mean-square difference between two images"
    diff = ImageChops.difference(im1, im2)
    h = diff.histogram()
    sq = (value * ((idx % 256) ** 2) for idx, value in enumerate(h))
    sum_of_squares = sum(sq)
    rms = math.sqrt(sum_of_squares / float(im1.size[0] * im1.size[1]))
    return rms




import os
from PIL import ImageFont
from PIL import Image
from PIL import ImageDraw
from sepya.utils import crop_out_identical_margins

folder_containing_current_file, _ = os.path.split(__file__)
ttf_font_folder = os.path.join(folder_containing_current_file, 'data/fonts/ttf/')

# DEFAULT_FONT_FILE = 'DejaVuSans-Bold.ttf'
DEFAULT_FONT_FILE = 'courier.ttf'

def local_font_files():
    return [f for f in os.listdir(ttf_font_folder) if f.endswith('.ttf')]


def text2img(text,
             color="black",
             bgcolor=None,
             font_file=DEFAULT_FONT_FILE,
             vspace=20,
             fontsize=50,
             width=600,
             x_margin=0,
             y_margin=0):
    """

    >>> img = text2img('asdf ' * 40, color='white', fontsize=40, x_margin=0, y_margin=0, width=600)
    >>> img.save(open('asdf.png', 'w'))
    :param text:
    :param color:
    :param bgcolor:
    :param fontfullpath:
    :param vspace:
    :param fontsize:
    :param width:
    :param x_margin:
    :param y_margin:
    :return:
    """

    if color in ['w', 'white']:
        color = "#FFF"
    elif color in ['k', 'black']:
        color = "#000"
    if bgcolor is None:
        if color == "#000":
            bgcolor = '#FFF'
        else:
            bgcolor = "#000"

    if font_file is not None and not os.path.isfile(font_file):
        font_file = os.path.join(ttf_font_folder, font_file)

    REPLACEMENT_CHARACTER = '\uFFFD'
    NEWLINE_REPLACEMENT_STRING = ' ' + REPLACEMENT_CHARACTER + ' '

    font = ImageFont.load_default() if font_file == None \
        else ImageFont.truetype(font_file, fontsize)
    text = text.replace('\n', NEWLINE_REPLACEMENT_STRING)

    lines = []
    line = ""

    for word in text.split():
        if word == REPLACEMENT_CHARACTER:  # give a blank line
            lines.append(line[1:])  # slice the white space in the begining of the line
            line = ""
            lines.append("")  # the blank line
        elif font.getsize(line + ' ' + word)[0] <= (width):
            line += ' ' + word
        else:  # start a new line
            lines.append(line[1:])  # slice the white space in the begining of the line
            line = ""

            # TODO: handle too long words at this point
            line += ' ' + word  # for now, assume no word alone can exceed the line width

    if len(line) != 0:
        lines.append(line[1:])  # add the last line

    line_height = font.getsize(text)[1]
    img_height = line_height * (len(lines) + 1)
    img_width = max([font.getsize(line)[0] for line in lines])
    print(img_width)

    img = Image.new("RGBA", (img_width, img_height), bgcolor)
    draw = ImageDraw.Draw(img)

    y = 0
    for line in lines:
        draw.text((0, y), line, color, font=font)
        y += vspace + line_height / 2

    img = crop_out_identical_margins(img, x_margin=x_margin, y_margin=y_margin)

    return img
